const std = @import("std");
const vk = @import("vulkan");
const sdl = @import("sdl2");
const VulkanRenderer = @import("vulkan_renderer.zig").VulkanRenderer;

pub fn main() !void {
    const window = try initWindow();
    defer sdl.quit();
    defer sdl.vulkan.unloadLibrary();
    defer window.destroy();

    var gpa = std.heap.GeneralPurposeAllocator(.{}){};
    defer _ = gpa.deinit();
    const allocator = gpa.allocator();

    var vulkan_renderer = try VulkanRenderer.init(window, allocator);
    defer vulkan_renderer.deinit();

    mainLoop: while (true) {
        while (sdl.pollEvent()) |ev| {
            switch (ev) {
                .quit => break :mainLoop,
                .key_down => |key| {
                    if (key.keycode == sdl.Keycode.escape) {
                        break :mainLoop;
                    }
                },
                else => {},
            }
        }

        try vulkan_renderer.draw();
    }
}

fn initWindow() !sdl.Window {
    sdl.init(.{ .timer = true, .video = true, .events = true }) catch {
        std.log.err("Failed to initialize SDL2", .{});
    };

    return try sdl.createWindow(
        "Vulkan Test",
        .{ .centered = {} },
        .{ .centered = {} },
        800,
        600,
        .{ .vis = .shown, .context = .vulkan },
    );
}
